# Top level Makefile that builds the Linaro cross compilers
#
# Copyright 2011  Linaro Limited
#

TOP := $(shell pwd)

# Top level product
PRODUCT ?= gcc-linaro
# Target this build compiles for
TARGET ?= arm-linux-gnueabi
# Host OS the toolchain will run on
HOST ?= linux
# Major version of the toolchain.  Pulled automatically from the
# .config file
VERSION = $(shell awk 'BEGIN {FS="[=\"-]"}; /^CT_CC_VERSION/ {print $$5}' \
                      samples/linaro-$(TARGET)/crosstool.config)
# Binary build identifier.  YYYYMMDD
SPIN = 20120326
# Development build identifier.  Normally +bzr1234
REVISION =

# Full name of this release.  Used in the tarball name
FULL = $(PRODUCT)-$(TARGET)-$(VERSION)-$(SPIN)$(REVISION)
# Tarball/directory name
DIRNAME = $(FULL)_$(HOST)

TARGETS = arm-linux-gnueabi arm-none-eabi
HOSTS = linux win32

BUILD = builds
HBUILD = $(BUILD)/$(TARGET)-$(HOST)
# Directory to put all stamp files
stamp = $(BUILD)/stamp/
# Sub build specific stamps
dstamp = $(stamp)$(TARGET)-$(HOST)-
INSTALL = $(HBUILD)/install
# Directory the build is composed into before taring
FINAL = $(HBUILD)/$(DIRNAME)
SYSROOT = $(FINAL)/$(TARGET)/libc
# List of files that overlap on case-insensitive filesystems
LINUX_INC = $(SYSROOT)/usr/include/linux
NETFILTER_RENAMES = \
	netfilter_ipv4/ipt_ECN \
	netfilter_ipv4/ipt_TTL \
	netfilter_ipv6/ip6t_HL \
	netfilter/xt_CONNMARK \
	netfilter/xt_DSCP \
	netfilter/xt_MARK \
	netfilter/xt_RATEEST \
	netfilter/xt_TCPMSS

# Subdirectory under share/doc to put the manuals
DOC_DIR = $(FINAL)/share/doc/$(PRODUCT)-$(TARGET)

all: $(stamp)init-dirs targets $(stamp)src

# The foreach expands to all combinations of TARGETS and HOSTS
targets: $(foreach t,$(TARGETS),$(HOSTS:%=$(stamp)$(t)-%))

$(stamp)init-dirs:
	install -d $(stamp) tarballs
	touch $@

# Build the local LSB environment
$(BUILD)/lsb/env.sh:
	$(TOP)/contrib/linaro/lsb/make-lsb.sh $(@D)

# Build the Linux version using the LSB compilers
$(stamp)%-linux: $(BUILD)/lsb/env.sh
	. $< && \
	PATH=$(TOP):$$PATH \
	make -f contrib/linaro/build.mk HOST=linux TARGET=$* go
	touch $@

# Build the win32 version using the just-built Linux compiler
$(stamp)%-win32: $(stamp)%-linux
	PATH=$(TOP):$(TOP)/$(BUILD)/$*-linux/install/bin:$$PATH \
	make -f contrib/linaro/build.mk HOST=win32 TARGET=$* go
	touch $@

# Build a tarball of everything that went into the build
$(stamp)src:
	rm -rf $(BUILD)/src/$(FULL)_src
	mkdir -p $(BUILD)/src/$(FULL)_src
	cp -afu tarballs/* $(BUILD)/src/$(FULL)_src
	cd $(BUILD)/src/$(FULL)_src && md5sum * > md5sum
	cd $(BUILD)/src && tar cjf $(FULL)_src.tar.bz2 $(FULL)_src
	touch $@

# Stage 2: target and host specific
go: $(dstamp)archive

INSTALLER = $(TOP)/contrib/linaro/win32installer
$(BUILD)/win32installer/env.sh:
	$(INSTALLER)/setup.sh $(@D)

# Host specific ways of tarballing up.  Note that the .zip.xz version
# is 33 % of the size of the plain .zip
$(stamp)$(TARGET)-win32-archive: $(dstamp)fixup $(BUILD)/win32installer/env.sh
	rm -f $(HBUILD)/$(DIRNAME).zip*
	cd $(HBUILD) && zip -r0q $(DIRNAME).zip $(DIRNAME)
	xz -f9 $(HBUILD)/$(DIRNAME).zip
	cd $(HBUILD) && zip -r9q $(DIRNAME).zip $(DIRNAME)
	# Create a Windows installer
	cp $(INSTALLER)/gccvar.bat $(HBUILD)/$(DIRNAME)/bin
	$(TOP)/$(BUILD)/win32installer/installjammer/installjammer     \
		-DBaseDir $(TOP)/$(HBUILD)/$(DIRNAME) -DDate $(SPIN)$(REVISION)  \
		-DVersion $(VERSION) -DImage $(INSTALLER)/Linaro_Green.gif \
		-DIcon $(INSTALLER)/Linaro_Sprinkles.gif                   \
		--output-dir $(TOP)/$(HBUILD)                              \
		--build $(INSTALLER)/$(TARGET).mpi

# Create a tarball and a xz compressed one for testing
$(stamp)$(TARGET)-linux-archive: $(dstamp)fixup
	cd $(HBUILD) && tar cf $(DIRNAME).tar $(DIRNAME)
	bzip2 -fk9 $(HBUILD)/$(DIRNAME).tar
	xz -fk9 $(HBUILD)/$(DIRNAME).tar

# Build
$(dstamp)build: $(HBUILD)/.config $(stamp)init-dirs
	cd $(HBUILD) && ln -fs ../../tarballs
	cd $(HBUILD) && DEB_HOST_MULTIARCH=$(TARGET) ct-ng build
	touch $@

# Fix up the full tree
$(dstamp)fixup: $(dstamp)$(TARGET)-$(HOST)-fixup $(dstamp)$(HOST)-fixup $(dstamp)$(TARGET)-fixup
	touch $@

# Fallback rule for no fixups
$(dstamp)%-fixup: $(dstamp)common-fixup
	true

# Common fixups that run first
$(dstamp)common-fixup: $(dstamp)build
	# Copy the install/ tree to a directory with the full name
	rm -rf $(FINAL)
	cp -a $(INSTALL) $(FINAL)
	# Remove host libiberty.a
	rm $(FINAL)/lib/libiberty.a
	# Remove the biarch symlinks
	find $(FINAL) -type l -name lib64 -exec rm {} \;
	find $(FINAL) -type l -name lib32 -exec rm {} \;
	# Remove the buildlog
	rm -f $(FINAL)/build.log.bz2
	# Remove the *-cc[.exe] symlink
	rm -f $(FINAL)/bin/*-cc*
	# Remove the populate script
	rm -f $(FINAL)/bin/*-populate
	# Remove the internal documentation
	rm -rf $(FINAL)/share/{doc,info,man}
	# .config shouldn't be executable
	chmod -x $(FINAL)/bin/*.config
	# Remove include/
	rm -rf $(FINAL)/include
	# Remove all libtool .la files (LP: #916671)
	find $(FINAL) -name "*.la" -exec rm {} \;
	# Copy the documentation across while splitting into html and
	# pdf directories
	mkdir -p $(DOC_DIR)/{html,pdf}
	cp -a $(INSTALL)/share/doc/*/*.pdf $(DOC_DIR)/pdf
	cp -a $(INSTALL)/share/*/*.pdf $(DOC_DIR)/pdf
	cp -a $(INSTALL)/share/doc/* $(DOC_DIR)/html
	rm -f $(DOC_DIR)/html/*.pdf
	cp -a $(INSTALL)/share/{man,info} $(DOC_DIR)
	rm -rf $(DOC_DIR)/*/{annotate,cppinternals,gccinstall,gccint,gdbint,stabs,libquadmath,standards}*
	# Fix any -real man pages
	for i in `ls $(DOC_DIR)/man/*/*-real*`; do \
		mv $$i `echo $$i | sed "s/-real//"`; done
	# Copy any Linaro specific documentation
	cp -af contrib/linaro/doc/* $(DOC_DIR)

# Host or target specific fixups

$(dstamp)arm-linux-gnueabi-win32-fixup: $(dstamp)common-fixup
	# Rename the netfilter header files
	for i in $(NETFILTER_RENAMES); do \
		mv $(LINUX_INC)/$${i}.h $(LINUX_INC)/$${i}_.h; \
	done

SECTIONS = -R .comment -R .note -R .debug_info -R .debug_aranges    \
	-R .debug_pubnames -R .debug_pubtypes -R .debug_abbrev      \
	-R .debug_line -R .debug_str -R .debug_ranges -R .debug_loc \
	-R .debug_types -R .debug_macinfo

$(dstamp)win32-fixup: $(dstamp)common-fixup
	# Fix the line endings of any text files
	find $(FINAL) -name "*.txt" -print -exec flip -m {} \;
	# Strip .comment, .note and .debug related sections
	find $(FINAL)/ -name \*.a \
		-exec $(TARGET)-objcopy $(SECTIONS) {} \;
	find $(FINAL)/ -name \*.o \
		-exec $(TARGET)-objcopy $(SECTIONS) {} \;
	# gdb document has two files index.html and Index.html. Change
	# Index.html to Index_.html since filename on windows is case insensitive.
	-mv $(DOC_DIR)/html/gdb/Index.html $(DOC_DIR)/html/gdb/Index_.html
	find $(DOC_DIR)/html/gdb -name "*.html" \
		-exec sed -i 's/\"Index\.html/\"Index_.html/' {} \;

$(dstamp)linux-fixup: $(dstamp)common-fixup
	# Strip .comment, .note and .debug related sections
	find $(FINAL)/ -name \*.a \
		-exec $(FINAL)/bin/$(TARGET)-objcopy $(SECTIONS) {} \;
	find $(FINAL)/ -name \*.o \
		-exec $(FINAL)/bin/$(TARGET)-objcopy $(SECTIONS) {} \;

# Settings to add just for Linux
LINUX_ADD = \
	CT_BUILD_USE_LSBCC=y

# Settings to add just for win32
WIN32_ADD = \
	CT_CANADIAN=y \
	CT_HOST=\"i586-mingw32msvc\"

WIN32_REMOVE = \
	CT_CROSS \
	CT_BUILD_USE_LSBCC \
	CT_CROSS_EXTRAS_pkg_config

# Build the Linux specific configuration
$(BUILD)/$(TARGET)-linux/.config: ct-ng
	mkdir -p $(@D)
	cd $(@D) && echo | ct-ng linaro-$(TARGET)
	for i in $(LINUX_ADD); do \
		echo $$i >> $@; \
	done
	cd $(@D) && echo | ct-ng oldconfig

# Build the Windows specific configuration
$(BUILD)/$(TARGET)-win32/.config: ct-ng
	mkdir -p $(@D)
	cd $(@D) && ct-ng linaro-$(TARGET)
	# The win32 changes are small.  Patch them here
	for i in $(WIN32_REMOVE); do \
		sed -i -r "s/^$$i.+//" $@; \
	done
	for i in $(WIN32_ADD); do \
		echo $$i >> $@; \
	done
	cd $(@D) && echo | ct-ng oldconfig

# Build ct-ng locally
ct-ng:
	./configure --local
	make -S MAKELEVEL=0

# Tidy up
clean:
	-$(MAKE) clean -S MAKELEVEL=0
	rm -rf $(BUILD) $(stamp)

# Grab the crosstool-NG version minus the +bzr
UPSTREAM_VERSION = $(firstword $(subst +, ,$(shell ./version.sh)))
RELEASE_VERSION = $(UPSTREAM_VERSION)-$(VERSION)-$(SPIN)
RELEASE_NAME = crosstool-ng-$(RELEASE_VERSION)
RELEASE_DIR = builds/release/$(RELEASE_NAME)

# Make the release tarball
release:
	rm -rf builds/release
	install -d builds/release
	bzr export $(RELEASE_DIR)
	# Patch the version number
	rm -f $(RELEASE_DIR)/version.sh
	echo $(RELEASE_VERSION) > $(RELEASE_DIR)/.version
	# Tar it up
	cd builds/release && tar cjf $(RELEASE_NAME).tar.bz2 $(RELEASE_NAME)
